//+------------------------------------------------------------------+
//|                                                  CWebRequest.mqh |
//|                                                   A.Lopatin 2017 |
//|                                              diver.stv@gmail.com |
//+------------------------------------------------------------------+
#property copyright "A.Lopatin 2017"
#property link      "diver.stv@gmail.com"
#property strict

//+-----------------------------------------------------------------------------+
//| WebQueryParameter is struct, that stores web-parameter as pair name = value |
//+-----------------------------------------------------------------------------+
struct WebQueryParameter
{
   public:
      string m_name;   // name of the parameter
      string m_value;  // value of the parameter
      
      WebQueryParameter(const string i_name, const string i_value)
      {
         this.m_name = i_name;
         this.m_value = i_value;
      }
      // Copy constructor
      WebQueryParameter(const WebQueryParameter& src)
      {
         this.m_name = src.m_name;
         this.m_value = src.m_value;
      }
      // Copy operator
      void operator=(const WebQueryParameter& src)
      {
         this.m_name = src.m_name;
         this.m_value = src.m_value;
      }
      // Default constructor
      WebQueryParameter()
      {
         this.m_name = "";
         this.m_value = "";
      }
      
      void Create(const string i_name, const string i_value)
      {
         this.m_name = i_name;
         this.m_value = i_value;
      }
};

//+------------------------------------------------------------------+
//|    CWebRequest is class-wrapper for web request                  |
//+------------------------------------------------------------------+
class CWebRequest
{
   public:
      //--- default constructor
      CWebRequest(void);
      CWebRequest(const string base_url);
      //--- sends POST query 
      string RequestPost(const string method, const WebQueryParameter& parameters[], string& out_head);
      //--- sends GET query
      string RequestGet(const string method, const WebQueryParameter& parameters[], string& out_head);
   protected:
      //--- string representation for basic url (for ex.: https://google.com")
      string m_baseURL; 
      //--- Returns  string representation of parameters
      virtual string BuildParameters(const WebQueryParameter& parameters[]);
      //--- Sends request for defined http method:GET/POST, with defined parameters
      virtual string SendRequest(const string http_method, const string method, const WebQueryParameter& parameters[], string& out_head); 
};

//+------------------------------------------------------------------+
//|    Constructor takes basic url as string                         |
//+------------------------------------------------------------------+
CWebRequest::CWebRequest(const string base_url)
{
   this.m_baseURL = base_url;
}

CWebRequest::CWebRequest(void)
{
   this.m_baseURL = "";
}

//+------------------------------------------------------------------+
//|    Returns string result of POST-query                           |
//|    method - string name of method                                |
//|    parameters - array of parameters for the query                |
//|    out_head - out string for head of server response             |
//+------------------------------------------------------------------+
string CWebRequest::RequestPost(const string method, const WebQueryParameter& parameters[], string& out_head)
{
    return this.SendRequest("POST", method, parameters, out_head);
}

//+------------------------------------------------------------------+
//|    Returns struct WebResponse result of GET-query                |
//|    method - string name of method                                |
//|    parameters - array of parameters for the query                |
//|    out_head - out string for head of server response             |
//+------------------------------------------------------------------+
string CWebRequest::RequestGet(const string method, const WebQueryParameter& parameters[], string& out_head)
{
    return this.SendRequest("GET", method, parameters, out_head);
}

//+------------------------------------------------------------------+
//|    Returns string result of HTTP- query                          |
//|    http_method - http method: GET of POST                        |
//|    method - string name of method                                |
//|    parameters - array of parameters for the query                |
//|    out_head - out string for head of server response             |
//+------------------------------------------------------------------+
string CWebRequest::SendRequest(const string http_method, const string method, const WebQueryParameter& parameters[], string& out_head)
{
    int    res;     
    char   data[]; 
    string url = "",
           data_str = this.BuildParameters(parameters),
           str_result = "";
    
    if( StringSubstr(this.m_baseURL, StringLen(this.m_baseURL)-1, 1) == "/" )
      url = StringConcatenate(this.m_baseURL, method);
    else
      url = StringConcatenate(this.m_baseURL, "/", method);
      
    ArrayResize(data, StringToCharArray(data_str, data, 0, WHOLE_ARRAY,CP_UTF8)-1);
    ResetLastError();
    #ifdef _DEBUG
    Print("CWebRequest::SendRequest(): ", url, "?", data_str);
    #endif
    res = WebRequest(http_method, url, NULL, NULL, 10000, data, ArraySize(data), data, str_result);
    if(res == -1) 
    { 
      Print("Error in WebRequest(). Error code: ", GetLastError(), " Result: ",  CharArrayToString(data));
    }
    
    out_head = str_result;
    
    return CharArrayToString(data);
}

//+------------------------------------------------------------------+
//|    Returns string representation of parameters for an query      |
//|    parameters - array of parameters for the query                |
//+------------------------------------------------------------------+
string CWebRequest::BuildParameters(const WebQueryParameter& parameters[])
{
   string query = "";
   int size = ArraySize(parameters);
   
   if( size <= 0 ) 
      return query;
   
   for(int i = 0; i < size; i++)
   {
      if( StringLen(parameters[i].m_name) < 1 )
         continue;
      if( i == 0 )
         query = StringConcatenate(parameters[i].m_name, "=", parameters[i].m_value);
      else
         query = StringConcatenate(query,"&", parameters[i].m_name, "=", parameters[i].m_value);
   }
     
   return query;
}